const std = @import("std");

pub const BingoData = struct {
    bingos: [][]const u8,
    arena: std.heap.ArenaAllocator,

    pub fn deinit(self: *BingoData) void {
        self.arena.deinit();
        self.* = undefined;
    }
};

pub fn loadFile(path: []const u8, count: usize) !BingoData {
    var arena = std.heap.ArenaAllocator.init(std.heap.c_allocator);

    const file = try std.fs.cwd().openFile(path, .{});
    defer file.close();
    const data = try file.readToEndAlloc(arena.allocator(), std.math.maxInt(usize));

    var bingos = std.ArrayList([]const u8).init(std.heap.c_allocator);
    defer bingos.deinit();
    var data_iter = std.mem.tokenize(u8, data, ",\n");
    while (data_iter.next()) |bingo| {
        const trimmed = std.mem.trim(u8, bingo, "\t\n ");
        if (trimmed.len == 0)
            continue;
        try bingos.append(trimmed);
    }

    var rng = std.rand.DefaultPrng.init(@bitCast(std.time.microTimestamp()));
    rng.random().shuffle([]const u8, bingos.items);

    const bingo_items = try arena.allocator().alloc([]const u8, count);

    var i: usize = 0;
    while (i < count) : (i += 1) {
        bingo_items[i] = if (bingos.items.len > i) bingos.items[i] else "";
    }

    return .{
        .bingos = bingo_items,
        .arena = arena,
    };
}
