const std = @import("std");
const c = @import("ffi.zig").c;
const gl = @import("gl.zig");
const Color = @import("Color.zig");
const GlState = @import("GlState.zig");

pub fn rectangle(
    vertex_buf: *std.ArrayList(f32),
    x1: f32,
    y1: f32,
    x2: f32,
    y2: f32,
    z: f32,
    color: Color,
) !void {
    // TODO: indexed drawing?
    // zig fmt: off
    const vertices = [_]f32{
        x1, y1, z, color.r, color.g, color.b, color.a,
        x1, y2, z, color.r, color.g, color.b, color.a,
        x2, y1, z, color.r, color.g, color.b, color.a,

        x1, y2, z, color.r, color.g, color.b, color.a,
        x2, y1, z, color.r, color.g, color.b, color.a,
        x2, y2, z, color.r, color.g, color.b, color.a,
    };
    // zig fmt: on

    try vertex_buf.appendSlice(&vertices);
}

pub fn copyVertices(vertices: []const f32) void {
    gl.bindBuffer(gl.ARRAY_BUFFER, GlState.state.vbo);
    gl.bufferData(
        gl.ARRAY_BUFFER,
        @intCast(vertices.len * @sizeOf(f32)),
        vertices.ptr,
        gl.DYNAMIC_DRAW,
    );
}

pub fn drawTriangles(vertices: []const f32) void {
    gl.useProgram(GlState.state.shader_program);
    copyVertices(vertices);
    gl.bindVertexArray(GlState.state.vao);
    gl.drawArrays(gl.TRIANGLES, 0, @intCast(vertices.len / 3));
}

pub fn loadShader(src: [:0]const u8, t: c_uint) !c_uint {
    const id = gl.createShader(t);
    gl.shaderSource(
        id,
        1,
        &[_][*:0]const u8{src},
        &[_]c_int{@intCast(src.len)},
    );
    gl.compileShader(id);
    try checkShaderCompileError(id);
    return id;
}

pub fn checkShaderCompileError(shader: c_uint) !void {
    var success: c_int = 0;
    gl.getShaderiv(shader, gl.COMPILE_STATUS, &success);
    if (success == 0) {
        var info_buf = std.mem.zeroes([512:0]u8);
        gl.getShaderInfoLog(shader, 512, null, &info_buf);
        std.log.err("compiling shader: {s}", .{&info_buf});
        return error.ShaderCompileError;
    }
}

pub fn checkProgramLinkError(program: c_uint) !void {
    var success: c_int = 0;
    gl.getProgramiv(program, gl.LINK_STATUS, &success);
    if (success == 0) {
        var info_buf = std.mem.zeroes([512:0]u8);
        gl.getProgramInfoLog(program, 512, null, &info_buf);
        std.log.err("linking program: {s}", .{&info_buf});
        return error.ProgramError;
    }
}
